# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4


"""This is helper module that contains functions
to easeup communication with seaweed-fs
"""

import requests

from pyseaweed.version import __version__


class Connection(object):
    def __init__(self, use_session=False):
        if use_session:
            self._conn = requests.Session()
        else:
            self._conn = requests

    def _prepare_headers(self, additional_headers=None, **kwargs):
        """Prepare headers for http communication.

        Return dict of header to be used in requests.

        Args:
            .. versionadded:: 0.3.2
                **additional_headers**: (optional) Additional headers
                to be used with request

        Returns:
            Headers dict. Key and values are string

        """
        user_agent = "pyseaweed/{version}".format(version=__version__)
        headers = {"User-Agent": user_agent}
        if additional_headers is not None:
            headers.update(additional_headers)
        return headers

    def head(self, url, *args, **kwargs):
        """Returns response to http HEAD
        on provided url
        """
        res = self._conn.head(url, headers=self._prepare_headers(**kwargs))
        if res.status_code == 200:
            return res
        return None

    def get_data(self, url, *args, **kwargs):
        """Gets data from url as text

        Returns content under the provided url as text

        Args:
            **url**: address of the wanted data

            .. versionadded:: 0.3.2
                **additional_headers**: (optional) Additional headers
                to be used with request

        Returns:
            string

        """
        res = self._conn.get(url, headers=self._prepare_headers(**kwargs))
        if res.status_code == 200:
            return res.text
        else:
            return None

    def get_raw_data(self, url, *args, **kwargs):
        """Gets data from url as bytes

        Returns content under the provided url as bytes
        ie. for binary data

        Args:
            **url**: address of the wanted data

            .. versionadded:: 0.3.2
                **additional_headers**: (optional) Additional headers
                to be used with request

        Returns:
            bytes

        """
        res = self._conn.get(url, headers=self._prepare_headers(**kwargs))
        if res.status_code == 200:
            return res.content
        else:
            return None

    def post_file(self, url, filename, file_stream, *args, **kwargs):
        """Uploads file to provided url.

        Returns contents as text

        Args:
            **url**: address where to upload file

            **filename**: Name of the uploaded file

            **file_stream**: file like object to upload

            .. versionadded:: 0.3.2
                **additional_headers**: (optional) Additional headers
                to be used with request

        Returns:
            string
        """
        res = self._conn.post(
            url,
            files={filename: file_stream},
            headers=self._prepare_headers(**kwargs),
        )
        if res.status_code == 200 or res.status_code == 201:
            return res.text
        else:
            return None

    def delete_data(self, url, *args, **kwargs):
        """Deletes data under provided url

        Returns status as boolean.

        Args:
            **url**: address of file to be deleted

            .. versionadded:: 0.3.2
                **additional_headers**: (optional) Additional headers
                to be used with request

        Returns:
            Boolean. True if request was successful. False if not.
        """
        res = self._conn.delete(url, headers=self._prepare_headers(**kwargs))
        if res.status_code == 200 or res.status_code == 202:
            return True
        else:
            return False



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from pyseaweed.seaweed import SeaweedFS  # noqa: F401
from pyseaweed.version import __version__

WeedFS = SeaweedFS  # for backward compatibilty

VERSION = __version__



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4


import unittest

from httmock import HTTMock

from pyseaweed.seaweed import SeaweedFS
from pyseaweed.utils import Connection


def response_content(url, request):
    return {"status_code": 200, "content": b"OK"}


def response_content_201(url, request):
    return {"status_code": 201, "content": b"OK"}


def response_content_202(url, request):
    return {"status_code": 202, "content": b"OK"}


def response_content_404(url, request):
    return {"status_code": 404, "content": b"NOK"}


class ReqTests(unittest.TestCase):
    def setUp(self):
        self.conn = Connection()
        pass

    def test_post_file(self):
        with HTTMock(response_content):
            r = self.conn.post_file(
                "http://utek.pl", "tests.py", open(__file__, "rb")
            )
            self.assertEqual(r, "OK")
        with HTTMock(response_content_201):
            r = self.conn.post_file(
                "http://utek.pl", "tests.py", open(__file__, "rb")
            )
            self.assertEqual(r, "OK")
        with HTTMock(response_content_404):
            r = self.conn.post_file(
                "http://utek.pl", "tests.py", open(__file__, "rb")
            )
            self.assertIsNone(r)

    def test_get_data(self):
        with HTTMock(response_content):
            r = self.conn.get_data("http://utek.pl")
            self.assertEqual(r, "OK")
        with HTTMock(response_content_404):
            r = self.conn.get_data("http://utek.pl")
            self.assertIsNone(r)

    def test_get_raw_data(self):
        with HTTMock(response_content):
            r = self.conn.get_raw_data("http://utek.pl")
            self.assertEqual(r, b"OK")
        with HTTMock(response_content_404):
            r = self.conn.get_raw_data("http://utek.pl")
            self.assertIsNone(r)

    def test_delete_data(self):
        with HTTMock(response_content):
            r = self.conn.delete_data("http://localhost")
            self.assertTrue(r)
        with HTTMock(response_content_202):
            r = self.conn.delete_data("http://localhost")
            self.assertTrue(r)
        with HTTMock(response_content_404):
            r = self.conn.delete_data("http://localhost")
            self.assertFalse(r)

    def test_prepare_headers(self):
        headers = self.conn._prepare_headers()
        self.assertIsInstance(headers, dict)
        for k, v in headers.items():
            self.assertIsInstance(k, str)
            self.assertIsInstance(v, str)

    def test_additional_headers(self):
        additional_headers = {"X-Test": "123"}
        kwargs = {"additional_headers": additional_headers}
        headers = self.conn._prepare_headers(**kwargs)
        self.assertIsInstance(headers, dict)
        self.assertIsNotNone(headers.get("X-Test"))
        with HTTMock(response_content):
            r = self.conn.post_file(
                "http://utek.pl",
                "tests.py",
                open(__file__, "rb"),
                additional_headers=additional_headers,
            )
            self.assertEqual(r, "OK")
            r = self.conn.get_data(
                "http://utek.pl", additional_headers=additional_headers
            )
            self.assertEqual(r, "OK")
            r = self.conn.get_raw_data(
                "http://utek.pl", additional_headers=additional_headers
            )
            self.assertEqual(r, b"OK")
            r = self.conn.delete_data(
                "http://localhost", additional_headers=additional_headers
            )
            self.assertTrue(r)


class SeaweedFSTests(unittest.TestCase):
    def setUp(self):
        self.seaweed = SeaweedFS()
        pass

    def test_repr(self):
        self.assertEqual(str(self.seaweed), "<SeaweedFS localhost:9333>")

    def test_exception(self):
        with HTTMock(response_content):
            with self.assertRaises(ValueError):
                self.seaweed.upload_file(stream=None, name="test.py")
            with self.assertRaises(ValueError):
                self.seaweed.upload_file(stream=open(__file__, "rb"))



#!/usr/bin/env python
# coding=utf-8
import os

from setuptools import find_packages, setup

__version__ = "0"

with open("pyseaweed/version.py") as f:
    exec(f.read())

LICENSE = (
    open(os.path.join(os.path.dirname(__file__), "LICENSE")).read().strip()
)

DESCRIPTION = (
    open(os.path.join(os.path.dirname(__file__), "README.rst")).read()
)

required_packages = ["requests", "six"]

setup(
    name="pyseaweed",
    version=__version__,
    description="Module to simplify usage of SeaweedFS in python.",
    author="Łukasz Bołdys",
    author_email="mail@utek.pl",
    license=LICENSE,
    long_description="Library to simplify communication with Seaweed-FS",
    url="https://github.com/utek/pyseaweed",
    packages=find_packages(),
    include_package_data=True,
    test_suite="pyseaweed",
    install_requires=required_packages,
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4


"""Main PySeaweed module. Contains SeaweedFS class
"""

import json
import os
import random
from collections import namedtuple

from pyseaweed.exceptions import BadFidFormat
from pyseaweed.utils import Connection


class SeaweedFS(object):
    master_addr = "localhost"
    master_port = 9333

    def __init__(
        self,
        master_addr="localhost",
        master_port=9333,
        use_session=False,
        use_public_url=True,
    ):
        """Creates SeaweedFS instance.

        Args:
            **master_addr**: Address of Seaweed-fs master server
                             (default: localhost)

            **master_port**: Weed-fs master server port (default: 9333)
            **use_session**: Use request.Session() for connections instead of
                             requests themselves. (default: False)
            **use_public_url**: If ``True``, all the requests will use
                             ``publicUrl`` link instead of ``url``.

        Returns:
            SeaweedFS instance.
        """
        self.master_addr = master_addr
        self.master_port = master_port
        self.conn = Connection(use_session)
        self.use_public_url = use_public_url

    def __repr__(self):
        return "<{0} {1}:{2}>".format(
            self.__class__.__name__, self.master_addr, self.master_port
        )

    def get_file(self, fid):
        """Get file from SeaweedFS.

        Returns file content. May be problematic for large files as content is
        stored in memory.

        Args:
            **fid**: File identifier <volume_id>,<file_name_hash>

        Returns:
            Content of the file with provided fid or None if file doesn't
            exist on the server

        .. versionadded:: 0.3.1
        """
        url = self.get_file_url(fid)
        return self.conn.get_raw_data(url)

    def get_file_url(self, fid, public=None):
        """
        Get url for the file

        :param string fid: File ID
        :param boolean public: public or internal url
        :rtype: string
        """
        try:
            volume_id, rest = fid.strip().split(",")
        except ValueError:
            raise BadFidFormat(
                "fid must be in format: <volume_id>,<file_name_hash>"
            )
        file_location = self.get_file_location(volume_id)
        if public is None:
            public = self.use_public_url
        volume_url = file_location.public_url if public else file_location.url
        url = "http://{volume_url}/{fid}".format(
            volume_url=volume_url, fid=fid
        )
        return url

    def get_file_location(self, volume_id):
        """
        Get location for the file,
        SeaweedFS volume is choosed randomly

        :param integer volume_id: volume_id
        :rtype: namedtuple `FileLocation` `{"public_url":"", "url":""}`
        """
        url = (
            "http://{master_addr}:{master_port}/"
            "dir/lookup?volumeId={volume_id}"
        ).format(
            master_addr=self.master_addr,
            master_port=self.master_port,
            volume_id=volume_id,
        )
        data = json.loads(self.conn.get_data(url))
        _file_location = random.choice(data["locations"])
        FileLocation = namedtuple("FileLocation", "public_url url")
        return FileLocation(_file_location["publicUrl"], _file_location["url"])

    def get_file_size(self, fid):
        """
        Gets size of uploaded file on SeaweedFS volume. For some type of files
        Gzip Compression might be applied.
        Or None if file doesn't exist.

        Args:
            **fid**: File identifier <volume_id>,<file_name_hash>

        Returns:
            Int or None
        """
        url = self.get_file_url(fid)
        res = self.conn.head(url)
        if res is not None:
            size = res.headers.get("content-length", None)
            if size is not None:
                return int(size)
        return None

    def file_exists(self, fid):
        """Checks if file with provided fid exists

        Args:
            **fid**: File identifier <volume_id>,<file_name_hash>

        Returns:
            True if file exists. False if not.
        """
        res = self.get_file_size(fid)
        if res is not None:
            return True
        return False

    def delete_file(self, fid):
        """
        Delete file from SeaweedFS

        :param string fid: File ID
        """
        url = self.get_file_url(fid)
        return self.conn.delete_data(url)

    def upload_file(self, path=None, stream=None, name=None, **kwargs):
        """
        Uploads file to SeaweedFS

        I takes either path or stream and name and upload it
        to SeaweedFS server.

        Returns fid of the uploaded file.

        :param string path:
        :param string stream:
        :param string name:
        :rtype: string or None

        """
        params = "&".join(["%s=%s" % (k, v) for k, v in kwargs.items()])
        url = "http://{master_addr}:{master_port}/dir/assign{params}".format(
            master_addr=self.master_addr,
            master_port=self.master_port,
            params="?" + params if params else "",
        )
        data = json.loads(self.conn.get_data(url))
        if data.get("error") is not None:
            return None
        post_url = "http://{url}/{fid}".format(
            url=data["publicUrl" if self.use_public_url else "url"],
            fid=data["fid"],
        )

        if path is not None:
            filename = os.path.basename(path)
            with open(path, "rb") as file_stream:
                res = self.conn.post_file(post_url, filename, file_stream)
        # we have file like object and filename
        elif stream is not None and name is not None:
            res = self.conn.post_file(post_url, name, stream)
        else:
            raise ValueError(
                "If `path` is None then *both* `stream` and `name` must not"
                " be None "
            )
        response_data = json.loads(res)
        if "size" in response_data:
            return data.get("fid")
        return None

    def vacuum(self, threshold=0.3):
        """
        Force garbage collection

        :param float threshold (optional): The threshold is optional, and
        will not change the default threshold.
        :rtype: boolean

        """
        url = (
            "http://{master_addr}:{master_port}/"
            "vol/vacuum?garbageThreshold={threshold}"
        ).format(
            master_addr=self.master_addr,
            master_port=self.master_port,
            threshold=threshold,
        )
        res = self.conn.get_data(url)
        if res is not None:
            return True
        return False

    @property
    def version(self):
        """
        Returns Weed-FS master version

        :rtype: string
        """
        url = "http://{master_addr}:{master_port}/dir/status".format(
            master_addr=self.master_addr, master_port=self.master_port
        )
        data = self.conn.get_data(url)
        response_data = json.loads(data)
        return response_data.get("Version")


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4


from __future__ import print_function

import os
import unittest

from pyseaweed.exceptions import BadFidFormat
from pyseaweed.seaweed import SeaweedFS


class FunctionalTests(unittest.TestCase):
    def setUp(self):
        self.seaweed = SeaweedFS()

    def test_head_file(self):
        _file = os.path.join(os.path.dirname(__file__), "../tox.ini")
        fid = self.seaweed.upload_file(_file)
        self.assertIsNotNone(fid)
        res = self.seaweed.get_file_size(fid)
        # Size is same or lower than file on disk
        self.assertTrue(res <= os.path.getsize(_file))
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)
        res = self.seaweed.get_file_size("3,123456790")
        self.assertIsNone(res)

    def test_upload_delete(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertIsNotNone(fid)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    def test_version(self):
        ver = self.seaweed.version
        self.assertIsNotNone(ver)

    def test_exists(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertTrue(self.seaweed.file_exists(fid))
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)
        self.assertFalse(self.seaweed.file_exists(fid))

    def test_upload_stream(self):
        with open(__file__, "rb") as stream:
            fid = self.seaweed.upload_file(stream=stream, name="test.py")
            self.assertIsNotNone(fid)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    # Test vacuum generated problems with Weed-FS on windows.
    # TODO: Investigate
    # def test_vacuum(self):
    #     res = self.seaweed.vacuum()
    #     self.assertTrue(res)
    def test_bad_fid(self):
        self.assertRaises(BadFidFormat, self.seaweed.get_file_url, ("a"))

    def test_get_file(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertIsNotNone(fid)
        file_content = self.seaweed.get_file(fid)
        self.assertIsNotNone(file_content)
        with open(__file__, "rb") as f:
            content = f.read()
        self.assertEqual(content, file_content)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    def test_get_wrong_file(self):
        file_content = self.seaweed.get_file("3,123456790")
        self.assertIsNone(file_content)


class FunctionalTestsSession(unittest.TestCase):
    def setUp(self):
        self.seaweed = SeaweedFS(use_session=True)

    def test_head_file(self):
        _file = os.path.join(os.path.dirname(__file__), "../tox.ini")
        fid = self.seaweed.upload_file(_file)
        self.assertIsNotNone(fid)
        res = self.seaweed.get_file_size(fid)
        # Size is same or lower than file on disk
        self.assertTrue(res <= os.path.getsize(_file))
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)
        res = self.seaweed.get_file_size("3,123456790")
        self.assertIsNone(res)

    def test_upload_delete(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertIsNotNone(fid)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    def test_version(self):
        ver = self.seaweed.version
        self.assertIsNotNone(ver)

    def test_exists(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertTrue(self.seaweed.file_exists(fid))
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)
        self.assertFalse(self.seaweed.file_exists(fid))

    def test_upload_stream(self):
        with open(__file__, "rb") as stream:
            fid = self.seaweed.upload_file(stream=stream, name="test.py")
            self.assertIsNotNone(fid)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    # Test vacuum generated problems with Weed-FS on windows.
    # TODO: Investigate
    # def test_vacuum(self):
    #     res = self.seaweed.vacuum()
    #     self.assertTrue(res)
    def test_bad_fid(self):
        self.assertRaises(BadFidFormat, self.seaweed.get_file_url, ("a"))

    def test_get_file(self):
        fid = self.seaweed.upload_file(__file__)
        self.assertIsNotNone(fid)
        file_content = self.seaweed.get_file(fid)
        self.assertIsNotNone(file_content)
        with open(__file__, "rb") as f:
            content = f.read()
        self.assertEqual(content, file_content)
        res = self.seaweed.delete_file(fid)
        self.assertTrue(res)

    def test_get_wrong_file(self):
        file_content = self.seaweed.get_file("3,123456790")
        self.assertIsNone(file_content)
